<?php

namespace Assetise\AssetType\Providers;

use Illuminate\Support\ServiceProvider;
use Assetise\AssetType\AssetTypes;

class AssetTypeServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = true;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'asset_type');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'asset_type');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerFacade();
        // $this->registerCommands();

        $this->app->register(\Assetise\AssetType\Providers\AuthServiceProvider::class);
        $this->app->register(\Assetise\AssetType\Providers\RouteServiceProvider::class);
        // $this->app->register(\Assetise\AssetType\Providers\EventServiceProvider::class);
        // $this->app->register(\Assetise\AssetType\Providers\WorkflowServiceProvider::class);
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->bind('assetise.asset_type', function($app)
        {
            return $this->app->make(AssetTypes::class);
        });
    }



    /**
     * Merges user's and asset_type's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'assetise.asset_type'
        );
        
        
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/asset_type.php', 'assetise.asset_type.asset_type'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                \Assetise\AssetType\Commands\AssetType::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['assetise.asset_type'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/' => config_path('assetise/asset_type')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/asset_type')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/asset_type')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
